@@ -33,7 +33,7 @@ MOVIT_HEAD=0
 MOVIT_REVISION="origin/shotcut"
 ENABLE_SWH_PLUGINS=0
 FFMPEG_HEAD=0
-FFMPEG_REVISION="origin/release/4.4"
+FFMPEG_REVISION="origin/release/5.1"
 FFMPEG_ADDITIONAL_OPTIONS=
 MLT_HEAD=1
 MLT_REVISION=
@@ -42,19 +42,22 @@ SHOTCUT_HEAD=1
 SHOTCUT_REVISION=
 SHOTCUT_VERSION=$(date '+%y.%m.%d')
 ENABLE_BIGSH0T=1
-BIGSH0T_HEAD=1
-BIGSH0T_REVISION=
+BIGSH0T_HEAD=0
+BIGSH0T_REVISION="8fe56f6d4e"
 ENABLE_ZIMG=1
-ZIMG_HEAD=1
-ZIMG_REVISION=
+ZIMG_HEAD=0
+ZIMG_REVISION="51c3c7f750c2af61955377faad56e3ba1b03589f"
 DAV1D_HEAD=0
-DAV1D_REVISION="0.9.2"
+DAV1D_REVISION="1.0.0"
 AOM_HEAD=0
-AOM_REVISION="v3.2.0"
+AOM_REVISION="v3.4.0"
 ENABLE_VMAF=1
 VMAF_HEAD=0
-VMAF_REVISION="v2.2.0"
-
+VMAF_REVISION="v2.3.1"
+ENABLE_GLAXNIMATE=1
+GLAXNIMATE_HEAD=0
+GLAXNIMATE_REVISION="v0.5.1"
+ENABLE_GOPRO2GPX=1
 
 # QT_INCLUDE_DIR="$(pkg-config --variable=prefix QtCore)/include"
 QT_INCLUDE_DIR=${QTDIR:+${QTDIR}/include}
@@ -75,17 +78,6 @@ DEBUG=0
 # We need to set LANG to C to avoid e.g. svn from getting to funky
 export LANG=C
 
-# User CFLAGS and LDFLAGS sometimes prevent more recent local headers.
-# Also, you can adjust some flags here.
-if [ "$DEBUG_BUILD" = "1" ]; then
-    export CFLAGS=
-    export CXXFLAGS=
-else
-    export CFLAGS=-DNDEBUG
-    export CXXFLAGS=-DNDEBUG
-fi
-export LDFLAGS=
-
 ################################################################################
 # FUNCTION SECTION
 ################################################################################
@@ -179,6 +171,12 @@ function to_key {
     vmaf)
       echo 12
     ;;
+    glaxnimate)
+      echo 13
+    ;;
+    gopro2gpx)
+      echo 14
+    ;;
     *)
       echo UNKNOWN
     ;;
@@ -312,6 +310,17 @@ function read_configuration {
 # This is where the configuration options for each subproject is assembled
 function set_globals {
   trace "Entering set_globals @ = $@"
+
+  # Set debug flags
+  if [ "$DEBUG_BUILD" = "1" ]; then
+    export CFLAGS=
+    export CXXFLAGS=
+  else
+    export CFLAGS=-DNDEBUG
+    export CXXFLAGS=-DNDEBUG
+  fi
+  export LDFLAGS=
+
   # Set convenience variables.
   if test 1 = "$ACTION_GET" ; then
     GET=1
@@ -353,6 +362,12 @@ function set_globals {
     if test "$ENABLE_VMAF" = 1 ; then
         SUBDIRS="vmaf $SUBDIRS"
     fi
+    if test "$ENABLE_GLAXNIMATE" = 1 ; then
+        SUBDIRS="$SUBDIRS glaxnimate"
+    fi
+    if test "$ENABLE_GOPRO2GPX" = 1 ; then
+        SUBDIRS="$SUBDIRS gopro2gpx"
+    fi
     SUBDIRS="$SUBDIRS mlt shotcut"
   fi
 
@@ -369,11 +384,9 @@ function set_globals {
   if [ "$ASAN_BUILD" = "1" ]; then
     ASAN_CFLAGS="-fsanitize=address -fno-omit-frame-pointer"
     ASAN_LDFLAGS="-lasan -fsanitize=address"
-    QMAKE_ASAN_FLAGS="QMAKE_CXXFLAGS+=-fsanitize=address QMAKE_CXXFLAGS+=-fno-omit-frame-pointer QMAKE_LFLAGS+=-fsanitize=address"
   else
     ASAN_CFLAGS=
     ASAN_LDFLAGS=
-    QMAKE_ASAN_FLAGS=
   fi
 
   debug "SUBDIRS = $SUBDIRS"
@@ -393,6 +406,8 @@ function set_globals {
   REPOLOCS[10]="https://code.videolan.org/videolan/dav1d.git"
   REPOLOCS[11]="https://aomedia.googlesource.com/aom"
   REPOLOCS[12]="https://github.com/Netflix/vmaf.git"
+  REPOLOCS[13]="https://gitlab.com/ddennedy/glaxnimate.git"
+  REPOLOCS[14]="https://github.com/ddennedy/gopro2gpx.git"
 
   # REPOTYPE Array holds the repo types. (Yes, this might be redundant, but easy for me)
   REPOTYPES[0]="git"
@@ -408,6 +423,8 @@ function set_globals {
   REPOTYPES[10]="git"
   REPOTYPES[11]="git"
   REPOTYPES[12]="git"
+  REPOTYPES[13]="git"
+  REPOTYPES[14]="git"
 
   # And, set up the revisions
   REVISIONS[0]=""
@@ -453,6 +470,11 @@ function set_globals {
   if test 0 = "$VMAF_HEAD" -a "$VMAF_REVISION" ; then
     REVISIONS[12]="$VMAF_REVISION"
   fi
+  REVISIONS[13]=""
+  if test 0 = "$GLAXNIMATE_HEAD" -a "$GLAXNIMATE_REVISION" ; then
+    REVISIONS[13]="$GLAXNIMATE_REVISION"
+  fi
+  REVISIONS[14]=""
 
   # Figure out the number of cores in the system. Used both by make and startup script
   CPUS=$(nproc)
@@ -500,14 +522,14 @@ function set_globals {
 
   #####
   # mlt
-  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DGPL=ON -DGPL3=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF $CMAKE_DEBUG_FLAG"
+  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DGPL=ON -DGPL3=ON -DMOD_GLAXNIMATE=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF $CMAKE_DEBUG_FLAG"
   CFLAGS_[1]="-I$FINAL_INSTALL_DIR/include $ASAN_CFLAGS $CFLAGS"
-  LDFLAGS_[1]="${LDFLAGS_[1]} -L$FINAL_INSTALL_DIR/lib $ASAN_LDFLAGS $LDFLAGS"
+  LDFLAGS_[1]="-L$FINAL_INSTALL_DIR/lib $ASAN_LDFLAGS $LDFLAGS"
 
-  ####
+  #####
   # frei0r
   CONFIG[2]="cmake -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DWITHOUT_GAVL=1 -DWITHOUT_OPENCV=1 -GNinja $CMAKE_DEBUG_FLAG"
-  CFLAGS_[2]="$CFLAGS -O2"
+  CFLAGS_[2]="$CFLAGS"
   LDFLAGS_[2]=$LDFLAGS
 
   #####
@@ -520,9 +542,9 @@ function set_globals {
 
   #####
   # shotcut
-  CONFIG[4]="$QTDIR/bin/qmake -r MLT_PATH=$FINAL_INSTALL_DIR PREFIX=$FINAL_INSTALL_DIR SHOTCUT_VERSION=$SHOTCUT_VERSION $QMAKE_DEBUG_FLAG $QMAKE_ASAN_FLAGS"
-  CFLAGS_[4]=$CFLAGS
-  LDFLAGS_[4]=$LDFLAGS
+  CONFIG[4]="cmake -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -D SHOTCUT_VERSION=$SHOTCUT_VERSION $CMAKE_DEBUG_FLAG"
+  CFLAGS_[4]="$ASAN_CFLAGS $CFLAGS"
+  LDFLAGS_[4]="$ASAN_LDFLAGS $LDFLAGS"
 
   #####
   # swh-plugins
@@ -578,6 +600,23 @@ function set_globals {
   fi
   CFLAGS_[12]=$CFLAGS
   LDFLAGS_[12]=$LDFLAGS
+
+  #####
+  # glaxnimate
+  CONFIG[13]="cmake -G Ninja -DCMAKE_PREFIX_PATH=$QTDIR -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR"
+  if [ "$DEBUG_BUILD" = "1" ]; then
+    CONFIG[13]="${CONFIG[13]} -DCMAKE_BUILD_TYPE=RelWithDebInfo"
+  else
+    CONFIG[13]="${CONFIG[13]} -DCMAKE_BUILD_TYPE=Release"
+  fi
+  CFLAGS_[13]="$ASAN_CFLAGS $CFLAGS"
+  LDFLAGS_[13]="$ASAN_LDFLAGS $LDFLAGS"
+
+  #####
+  # gopro2gpx
+  CONFIG[14]="cmake -G Ninja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG"
+  CFLAGS_[14]="$CFLAGS"
+  LDFLAGS_[14]="$LDFLAGS"
 }
 
 ######################################################################
@@ -727,9 +766,9 @@ function get_subproject {
           # No git repo
           debug "No git repo, need to check out"
           feedback_status "Cloning git sources for $1"
-          cmd git --no-pager clone --quiet $REPOLOC || die "Unable to git clone source for $1 from $REPOLOC"
+          cmd git --no-pager clone --quiet --recurse-submodules $REPOLOC || die "Unable to git clone source for $1 from $REPOLOC"
           cmd cd $1 || die "Unable to change to directory $1"
-          cmd git checkout $REVISION || die "Unable to git checkout $REVISION"
+          cmd git checkout --recurse-submodules $REVISION || die "Unable to git checkout $REVISION"
       fi
   elif test "svn" = "$REPOTYPE" ; then
       # Create subdir if not exist
@@ -855,7 +894,7 @@ function configure_compile_install_subproject {
 
   # Special hack for mlt
   if test "mlt" = "$1"; then
-    export CXXFLAGS="$CFLAGS -std=c++11"
+    export CXXFLAGS="$CFLAGS -std=c++11 -D_XOPEN_SOURCE=700"
   fi
 
   # Special hack for movit
@@ -898,7 +937,7 @@ function configure_compile_install_subproject {
   feedback_status Building $1 - this could take some time
   if test "movit" = "$1" ; then
     cmd make -j$MAKEJ libmovit.la || die "Unable to build $1"
-  elif test "frei0r" = "$1" -o "bigsh0t" = "$1" -o "aom" = "$1" -o "mlt" = "$1"; then
+  elif test "frei0r" = "$1" -o "bigsh0t" = "$1" -o "aom" = "$1" -o "mlt" = "$1" -o "shotcut" = "$1" -o "glaxnimate" = "$1" -o "gopro2gpx" = "$1"; then
     cmd ninja -j $MAKEJ || die "Unable to build $1"
   elif test "dav1d" = "$1"; then
     cmd ninja -C builddir -j $MAKEJ || die "Unable to build $1"
@@ -914,15 +953,12 @@ function configure_compile_install_subproject {
   export LD_LIBRARY_PATH=`lookup LD_LIBRARY_PATH_ $1`
   log "LD_LIBRARY_PATH=$LD_LIBRARY_PATH"
   if test "shotcut" = "$1" ; then
-    cmd make install
-    cmd install -c -m 755 src/shotcut.exe "$FINAL_INSTALL_DIR"
+    cmd ninja install
     cmd install -c COPYING "$FINAL_INSTALL_DIR"
     cmd install -c packaging/windows/shotcut.nsi "$FINAL_INSTALL_DIR"/..
     cmd sed -i "s/YY.MM.DD/$SHOTCUT_VERSION/" "$FINAL_INSTALL_DIR"/../shotcut.nsi
     cmd install -d "$FINAL_INSTALL_DIR"/share/translations
     cmd install -p -c translations/*.qm "$FINAL_INSTALL_DIR"/share/translations
-    cmd install -d "$FINAL_INSTALL_DIR"/share/shotcut
-    cmd cp -a src/qml "$FINAL_INSTALL_DIR"/share/shotcut
   elif test "frei0r" = "$1" -o "aom" = "$1" -o "mlt" = "$1"; then
     cmd ninja install || die "Unable to install $1"
   elif test "bigsh0t" = "$1" ; then
@@ -933,6 +969,11 @@ function configure_compile_install_subproject {
     cmd ninja install -C libvmaf/build || die "Unable to install $1"
     cmd install -d "$FINAL_INSTALL_DIR"/share/vmaf
     cmd install -p -c model/*.json "$FINAL_INSTALL_DIR"/share/vmaf || die "Unable to install $1"
+  elif test "glaxnimate" = "$1"; then
+    cmd ninja translations || die "Unable to build translations for $1"
+    cmd ninja install || die "Unable to install $1"
+  elif test "gopro2gpx" = "$1" ; then
+    cmd install -p -c gopro2gpx "$FINAL_INSTALL_DIR" || die "Unable to install $1"
   elif test "$MYCONFIG" != "" ; then
     cmd make install || die "Unable to install $1"
   fi
@@ -1054,12 +1095,14 @@ function deploy
   log Reorganizing installed files
   cmd mv bin/*.dll .
   cmd mv lib/libaom.dll .
+  cmd mv lib/libCuteLogger.dll .
   if [ "$SDK" = "1" ]; then
     cmd mv bin/*.exe .
   else
     cmd mv bin/ffmpeg.exe .
     cmd mv bin/ffplay.exe .
     cmd mv bin/ffprobe.exe .
+    cmd mv bin/glaxnimate.exe .
     cmd rm -rf bin include etc man manifest src *.txt
     cmd rm lib/*
     cmd rm -rf lib/cmake lib/pkgconfig lib/gdk-pixbuf-2.0 lib/glib-2.0 lib/gtk-2.0
@@ -1089,8 +1132,18 @@ function deploy
   for lib in lib/{frei0r-1,ladspa,mlt}/*.dll; do
     bundle_dlls "$lib"
   done
+  bundle_dlls glaxnimate.exe
   cmd rm *.bundled
 
+  log Copying some DLLs and python libraries for Glaxnimate
+  cmd cp -p /mingw64/bin/libpython3.10.dll python310.dll
+  cmd cp -p "$SOURCE_DIR"/glaxnimate/external/Qt-Color-Widgets/libQtColorWidgets.dll .
+  cmd mkdir -p share/glaxnimate/glaxnimate/pythonhome/lib/python
+  cmd cp -r /mingw64/lib/python3.10/*.py \
+            /mingw64/lib/python3.10/lib-dynload/* \
+            /mingw64/lib/python3.10/{json,collections,encodings,logging,urllib} \
+      share/glaxnimate/glaxnimate/pythonhome/lib/python
+
   cmd cp -p "$HOME"/bin/*.dll .
   cmd cp -p "$QTDIR"/bin/{libEGL,libGLESv2,d3dcompiler_47}.dll .
 
@@ -1194,9 +1247,8 @@ function deploy
     pushd .
     clean_dirs
     popd
-    log Copying src
-    cmd rm -rf src 2> /dev/null
-    cmd cp -a $SOURCE_DIR .
+    log Moving src
+    cmd mv $SOURCE_DIR .
 
     log Creating archive
     cmd cd ..
