@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2021 Meltytech, LLC
+ * Copyright (c) 2013-2022 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,25 +21,28 @@
 #include <QDockWidget>
 #include <QQuickWidget>
 #include <QApplication>
+#include <QTimer>
+#include <QDateTime>
 #include "models/markersmodel.h"
 #include "models/multitrackmodel.h"
 #include "sharedframe.h"
+#include "jobs/ffmpegjob.h"
 
-namespace Ui {
-class TimelineDock;
-}
 namespace Timeline {
 class UpdateCommand;
 class TrimCommand;
 }
 class UndoHelper;
+class QMenu;
 
 class TimelineDock : public QDockWidget
 {
     Q_OBJECT
     Q_PROPERTY(int position READ position WRITE setPosition NOTIFY positionChanged)
     Q_PROPERTY(int currentTrack READ currentTrack WRITE setCurrentTrack NOTIFY currentTrackChanged)
-    Q_PROPERTY(QVariantList selection READ selectionForJS WRITE setSelectionFromJS NOTIFY selectionChanged)
+    Q_PROPERTY(QVariantList selection READ selectionForJS WRITE setSelectionFromJS NOTIFY
+               selectionChanged)
+    Q_PROPERTY(bool isRecording READ isRecording NOTIFY isRecordingChanged)
 
 public:
     explicit TimelineDock(QWidget *parent = 0);
@@ -50,50 +53,73 @@ public:
         TrimOutPoint
     };
 
-    MultitrackModel* model() { return &m_model; }
-    MarkersModel* markersModel() { return & m_markersModel; }
-    int position() const { return m_position; }
+    MultitrackModel *model()
+    {
+        return &m_model;
+    }
+    MarkersModel *markersModel()
+    {
+        return & m_markersModel;
+    }
+    int position() const
+    {
+        return m_position;
+    }
     void setPosition(int position);
-    Mlt::ClipInfo* getClipInfo(int trackIndex, int clipIndex);
+    Mlt::ClipInfo *getClipInfo(int trackIndex, int clipIndex);
     Mlt::Producer producerForClip(int trackIndex, int clipIndex);
     int clipIndexAtPlayhead(int trackIndex = -1);
     int clipIndexAtPosition(int trackIndex, int position);
-    void chooseClipAtPosition(int position, int& trackIndex, int& clipIndex);
+    void chooseClipAtPosition(int position, int &trackIndex, int &clipIndex);
     void setCurrentTrack(int currentTrack);
     int currentTrack() const;
     int clipCount(int trackIndex) const;
-    void setSelectionFromJS(const QVariantList& list);
-    void setSelection(QList<QPoint> selection = QList<QPoint>(), int trackIndex = -1, bool isMultitrack = false);
+    void setSelectionFromJS(const QVariantList &list);
+    void setSelection(QList<QPoint> selection = QList<QPoint>(), int trackIndex = -1,
+                      bool isMultitrack = false);
     QVariantList selectionForJS() const;
     const QList<QPoint> selection() const;
     const QVector<QUuid> selectionUuids();
     void saveAndClearSelection();
-    void restoreSelection();
+    Q_INVOKABLE void restoreSelection();
     void selectClipUnderPlayhead();
     int centerOfClip(int trackIndex, int clipIndex);
     bool isTrackLocked(int trackIndex) const;
     void trimClipAtPlayhead(TrimLocation location, bool ripple);
-    bool isRipple() const;
-    Q_INVOKABLE bool isMultitrackSelected() const { return m_selection.isMultitrackSelected; }
-    Q_INVOKABLE int selectedTrack() const { return m_selection.selectedTrack; }
-    Q_INVOKABLE bool isFloating() const { return QDockWidget::isFloating(); }
-    Q_INVOKABLE void copyToSource();
+    Q_INVOKABLE bool isMultitrackSelected() const
+    {
+        return m_selection.isMultitrackSelected;
+    }
+    Q_INVOKABLE int selectedTrack() const
+    {
+        return m_selection.selectedTrack;
+    }
+    Q_INVOKABLE bool isFloating() const
+    {
+        return QDockWidget::isFloating();
+    }
     Q_INVOKABLE static void openProperties();
     void emitSelectedChanged(const QVector<int> &roles);
-    void replaceClipsWithHash(const QString& hash, Mlt::Producer& producer);
+    void replaceClipsWithHash(const QString &hash, Mlt::Producer &producer);
+    Q_INVOKABLE void recordAudio();
+    Q_INVOKABLE void stopRecording();
+    bool isRecording() const
+    {
+        return m_isRecording;
+    }
 
 signals:
     void currentTrackChanged();
     void selectionChanged();
     void seeked(int position);
     void positionChanged();
-    void clipOpened(Mlt::Producer* producer);
-    void dragging(const QPointF& pos, int duration);
+    void clipOpened(Mlt::Producer *producer);
+    void dragging(const QPointF &pos, int duration);
     void dropped();
     void dropAccepted(const QString &xml);
     void fadeInChanged(int duration);
     void fadeOutChanged(int duration);
-    void selected(Mlt::Producer* producer);
+    void selected(Mlt::Producer *producer);
     void clipClicked();
     void showStatusMessage(QString);
     void clipCopied();
@@ -104,16 +130,20 @@ signals:
     void zoomIn();
     void zoomOut();
     void zoomToFit();
-    void resetZoom();
-    void makeTracksShorter();
-    void makeTracksTaller();
+    void setZoom(double value);
     void markerRangesChanged();
     void markerSeeked(int markerIndex);
+    void isRecordingChanged(bool);
+    void multitrackSelected();
+    void warnTrackLocked(int trackIndex);
+    void refreshWaveforms();
+    void updateThumbnails(int trackIndex, int clipIndex);
 
 public slots:
     void addAudioTrack();
     void addVideoTrack();
-    void onShowFrame(const SharedFrame& frame);
+    void alignSelectedClips();
+    void onShowFrame(const SharedFrame &frame);
     void onSeeked(int position);
     void append(int trackIndex);
     void remove(int trackIndex, int clipIndex);
@@ -125,7 +155,7 @@ public slots:
     void selectTrackHead(int trackIndex);
     void selectMultitrack();
     void copy(int trackIndex, int clipIndex);
-    void setTrackName(int trackIndex, const QString& value);
+    void setTrackName(int trackIndex, const QString &value);
     void toggleTrackMute(int trackIndex);
     void toggleTrackHidden(int trackIndex);
     void setTrackComposite(int trackIndex, bool composite);
@@ -136,7 +166,7 @@ public slots:
     bool trimClipOut(int trackIndex, int clipIndex, int delta, bool ripple);
     void insert(int trackIndex, int position = -1, const QString &xml = QString(), bool seek = true);
     void overwrite(int trackIndex, int position = -1, const QString &xml = QString(), bool seek = true);
-    void appendFromPlaylist(Mlt::Playlist* playlist, bool skipProxy);
+    void appendFromPlaylist(Mlt::Playlist *playlist, bool skipProxy);
     void splitClip(int trackIndex = -1, int clipIndex = -1);
     void fadeIn(int trackIndex, int clipIndex = -1, int duration = -1);
     void fadeOut(int trackIndex, int clipIndex = -1, int duration = -1);
@@ -148,40 +178,56 @@ public slots:
     void insertAudioTrack();
     void insertVideoTrack();
     void removeTrack();
-    void onProducerChanged(Mlt::Producer*);
+    void moveTrack(int fromTrackIndex, int toTrackIndex);
+    void moveTrackUp();
+    void moveTrackDown();
+    void onProducerChanged(Mlt::Producer *);
     void emitSelectedFromSelection();
     void remakeAudioLevels(int trackIndex, int clipIndex, bool force = true);
     void commitTrimCommand();
-    void onRowsInserted(const QModelIndex& parent, int first, int last);
-    void onRowsRemoved(const QModelIndex& parent, int first, int last);
+    void onRowsInserted(const QModelIndex &parent, int first, int last);
+    void onRowsRemoved(const QModelIndex &parent, int first, int last);
+    void onRowsMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination,
+                     int row);
     void detachAudio(int trackIndex, int clipIndex);
     void selectAll();
+    void selectAllOnCurrentTrack();
     bool blockSelection(bool block);
     void onProducerModified();
-    void replace(int trackIndex, int clipIndex, const QString& xml = QString());
+    void replace(int trackIndex, int clipIndex, const QString &xml = QString());
     void createOrEditMarker();
+    void createOrEditSelectionMarker();
     void createMarker();
     void editMarker(int markerIndex);
     void deleteMarker(int markerIndex = -1);
     void seekNextMarker();
     void seekPrevMarker();
+    void onFilterModelChanged();
+    void trimClipIn(bool ripple = false);
+    void trimClipOut(bool ripple = false);
 
 protected:
-    void dragEnterEvent(QDragEnterEvent* event);
-    void dragMoveEvent(QDragMoveEvent* event);
-    void dragLeaveEvent(QDragLeaveEvent* event);
-    void dropEvent(QDropEvent* event);
+    void dragEnterEvent(QDragEnterEvent *event);
+    void dragMoveEvent(QDragMoveEvent *event);
+    void dragLeaveEvent(QDragLeaveEvent *event);
+    void dropEvent(QDropEvent *event);
     bool event(QEvent *event);
-    void keyPressEvent(QKeyEvent* event);
-    void keyReleaseEvent(QKeyEvent* event);
+    void keyPressEvent(QKeyEvent *event);
+    void keyReleaseEvent(QKeyEvent *event);
 
 private:
     bool isBlank(int trackIndex, int clipIndex);
-    void pulseLockButtonOnTrack(int trackIndex);
+    bool clipsAreSelected();
+    bool blankIsSelected();
+    bool isTransition(int trackIndex, int clipIndex);
     void emitNonSeekableWarning();
-    void addTrackIfNeeded(int mltTrackIndex, Mlt::Producer* srcTrack);
+    void addTrackIfNeeded(int mltTrackIndex, Mlt::Producer *srcTrack);
+    void setupActions();
+    bool isMultitrackValid()
+    {
+        return m_model.tractor() && !m_model.trackList().empty();
+    }
 
-    Ui::TimelineDock *ui;
     QQuickWidget m_quickView;
     MultitrackModel m_model;
     MarkersModel m_markersModel;
@@ -194,12 +240,23 @@ private:
         bool isMultitrackSelected;
     };
     Selection m_selection;
-    Selection m_savedSelection;
+    int m_savedSelectedTrack;
+    bool m_savedIsMultitrackSelected;
+    QVector<QUuid> m_savedSelectionUuids;
     QScopedPointer<Timeline::TrimCommand> m_trimCommand;
     QScopedPointer<UndoHelper> m_undoHelper;
     int m_trimDelta;
     int m_transitionDelta;
     bool m_blockSetSelection;
+    bool m_isRecording {false};
+    QScopedPointer<AbstractJob> m_recordJob;
+    QTimer m_recordingTimer;
+    QDateTime m_recordingTime;
+    int m_recordingTrackIndex;
+    int m_recordingClipIndex;
+    int m_currentTrack {0};
+    QMenu *m_mainMenu;
+    QMenu *m_clipMenu;
 
 private slots:
     void load(bool force = false);
@@ -208,12 +265,18 @@ private slots:
     void selectClip(int trackIndex, int clipIndex);
     void onMultitrackClosed();
     void reloadTimelineMarkers();
+    void onRecordStarted();
+    void updateRecording();
+    void onRecordFinished(AbstractJob *, bool);
+    void onWarnTrackLocked();
+    void onTimelineRightClicked();
+    void onClipRightClicked();
 };
 
 class TimelineSelectionBlocker
 {
 public:
-    TimelineSelectionBlocker(TimelineDock& timeline)
+    TimelineSelectionBlocker(TimelineDock &timeline)
         : m_timelineDock(timeline)
     {
         m_timelineDock.blockSelection(true);
@@ -225,7 +288,7 @@ public:
     }
 
 private:
-    TimelineDock& m_timelineDock;
+    TimelineDock &m_timelineDock;
 };
 
 #endif // TIMELINEDOCK_H
